import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.io.*;
import java.net.*;
import java.util.*;
import jrobocom.*;

/***************************************************************
 *The class TestApplet is a sample applet for showing how
 *to use jrobocom to Control a local device from a webpage.
 *
 *@see <img src="../source.gif"><a href="TestApplet/TestApplet.java">java-source-code</a>
 *@see <img src="../source.gif"><a href="../index.html" target="_parent">jrobocom help docs</a>
 *
 *@version 0.1, 04/2004
 *@author Gerald Kogler
****************************************************************/
public class TestApplet extends Applet implements RobotJavaListener, MouseListener
{
	private static boolean DEBUG=true;
	private RobotEventAdapter robAdapter;
	private String host;
	private TextArea sentMessages = new TextArea(),
				receivedMessages = new TextArea();
	private TextField status = new TextField();
	private int 	APP_WIDTH = 170,
			APP_HEIGHT = APP_WIDTH;
	private Button 	upButton = new Button("move UP"),
				downButton = new Button("move DOWN"),
				leftButton = new Button("move LEFT"),
				rightButton = new Button("move RIGHT"),
				connectButton = new Button("CONNECT");
	private boolean 	online = false,
				connecting = true,
				free = false,
				noRobot = false;

  /*************************************************************************
   *Constructor for the ControlApplet.
  *************************************************************************/
	public TestApplet(){}

  /*************************************************************************
   *Inits the ControlApplet.
  *************************************************************************/
	public void init()
	{
		//set textfields not editable
		sentMessages.setEditable(false);
		sentMessages.setRows(5);
		receivedMessages.setEditable(false);
		receivedMessages.setRows(5);
		status.setEditable(false);

		//add MouseListeners to buttons
		upButton.addMouseListener(this);
		upButton.setEnabled(false);
		downButton.addMouseListener(this);
		downButton.setEnabled(false);
		leftButton.addMouseListener(this);
		leftButton.setEnabled(false);
		rightButton.addMouseListener(this);
		rightButton.setEnabled(false);
		connectButton.addMouseListener(this);

		//Set the layout manager
	        setLayout(new java.awt.GridLayout(0,1));

	        //Add textfields and buttons to the applet.
        	add(upButton);
        	add(downButton);
        	add(leftButton);
        	add(rightButton);
		add(status);
        	add(connectButton);
        	add(sentMessages);
		add(receivedMessages);
		validate();

		//get host applet running on
		host = this.getCodeBase().getHost();
		System.out.println("Applet on Host: "+host);

		//start RobotEventAdapter for receiving messages
		robAdapter = new RobotEventAdapter(host, this);
		robAdapter.start();

		this.getParameters();
		//this.addMouseListener(this);
	}

  /*************************************************************************
   *Reads parameters from html-file.
  *************************************************************************/
	private void getParameters()
	{
		 //read applet width
    String appWidthString = getParameter("WIDTH");
    if (appWidthString != null) {
      try { APP_WIDTH = Integer.parseInt(appWidthString);}
      catch (NumberFormatException e) {}
    }

    //read applet height
    String appHeightString = getParameter("HEIGHT");
    if (appHeightString != null) {
      try { APP_HEIGHT = Integer.parseInt(appHeightString);}
      catch (NumberFormatException e) {}
    }
	}

  /*************************************************************************
   *Method for Mousehandling.
	 *
	 *@param ev MouseEvent to handle.
  *************************************************************************/
	public void mouseClicked(MouseEvent ev){}

  /*************************************************************************
   *Method for Mousehandling.
	 *
	 *@param ev MouseEvent to handle.
  *************************************************************************/
	public void mousePressed(MouseEvent ev)
	{
		if (ev.getComponent() == upButton){
			this.sendEventToRobot("JROBO UP "+host);
		}
		else if (ev.getComponent() == downButton){
			this.sendEventToRobot("JROBO DOWN "+host);
		}
		else if (ev.getComponent() == leftButton){
			this.sendEventToRobot("JROBO LEFT "+host);
		}
		else if (ev.getComponent() == rightButton){
			this.sendEventToRobot("JROBO RIGHT "+host);
		}
		else if (ev.getComponent() == connectButton){
			if (online)
				this.sendEventToRobot("JROBO DISCONNECT "+host);
			else
				this.sendEventToRobot("JROBO CONNECT "+host);
		}
	}

  /*************************************************************************
   *Method for Mousehandling.
	 *
	 *@param ev MouseEvent to handle.
  *************************************************************************/
	public void mouseReleased(MouseEvent ev)
	{
		if (ev.getComponent() == upButton){
			this.sendEventToRobot("JROBO STOP "+host);
		}
		else if (ev.getComponent() == downButton){
			this.sendEventToRobot("JROBO STOP "+host);
		}
		else if (ev.getComponent() == leftButton){
			this.sendEventToRobot("JROBO STOP "+host);
		}
		else if (ev.getComponent() == rightButton){
			this.sendEventToRobot("JROBO STOP "+host);
		}
	}

  /*************************************************************************
   *Method for Mousehandling.
	 *
	 *@param ev MouseEvent to handle.
  *************************************************************************/
	public void mouseEntered(MouseEvent ev){}

	public void mouseExited(MouseEvent ev){}


	/****************************************
	*Sends the command to init a remote-control.
	****************************************/
	private void sendInit()
	{
		this.sendEventToRobot("JROBO INIT "+host);
	}

  /*************************************************************************
   *Method for sending commands to robot.
	 *
	 *@param cmd Command to send to robot.
  *************************************************************************/
	private void sendEventToRobot(String cmd)
	{
		if (DEBUG) System.out.println("send to robot: "+cmd);
		robAdapter.send(cmd);
		sentMessages.append(cmd+"\n");
	}

	/****************************************
  *Method for receiving commands from robot.
	*
	*@param cmd Command received from robot.
 	****************************************/
	public void cmdReceived(String cmd)
	{
		if (cmd != null){
			if (DEBUG) System.out.println(">>> ACTION: "+cmd);
			if (cmd.equals("JROBO ONLINE")){
				online = true;
				upButton.setEnabled(true);
				downButton.setEnabled(true);
				leftButton.setEnabled(true);
				rightButton.setEnabled(true);
				status.setText("CONNECTED");
				connectButton.setLabel("DISCONNECT");
				receivedMessages.append("JROBO ONLINE"+"\n");
			}
			else if (cmd.equals("JROBO OFFLINE")){
				online = false;
				connecting = false;
				upButton.setEnabled(false);
				downButton.setEnabled(false);
				leftButton.setEnabled(false);
				rightButton.setEnabled(false);
				connectButton.setLabel("CONNECT");
				receivedMessages.append("JROBO OFFLINE"+"\n");
			}
			else if (cmd.equals("JROBO FREE")){
				connecting = false;
				free = true;
				status.setText("you CAN connect - no other user connected");
				receivedMessages.append("JROBO FREE"+"\n");
			}
			else if (cmd.equals("JROBO NOTFREE")){
				connecting = false;
				free = false;
				status.setText("you CAN'T connect - other user connected");
				receivedMessages.append("JROBO NOTFREE"+"\n");
			}
			else if (cmd.equals("JROBO OUT")){
				connecting = false;
				free = false;
				noRobot = true;
				receivedMessages.append("JROBO OUT"+"\n");
				status.setText("you CAN'T connect - no robots connected");
				connectButton.setEnabled(false);
			}
		}
	}
}
